C S B Singh and H K D H Bhadeshia, University of Cambridge
C June 1997
C
C Based on paper by Singh and Bhadeshia, submitted to 
C Materials Science and Engineering A, 1997
C
C Program predicts the bainite plate thickness in low-alloy steels.
C It does not give error bars, which are discussed in the original paper.
C  
C
C To use the output from neural network analysis
C  of the toughness of steel weld deposits
C
C The coefficients, in the form of weights are stored in a file
C  called WEIGHTS. 
C
C AW contains the input variables (unnormalised) 
C X contains normalized values of the input variables AW
C W1 and W2 contain the weights
C THETA1 and THETA2 are the biases
C Y is the output (normalized)
C AMIN and AMAX contain the minimum and maximum unnormalized values of the
C input variables and of the output
C

      IMPLICIT REAL*8(A-H,K-Z), INTEGER(I,J)
      DOUBLE PRECISION W1(4,14), W2(4),THETA1(14),
     &  AMIN(15),AMAX(15),AW(15)
C
      WRITE(*,*) 'Transformation Temperature (centigrade)?' 
      READ(*,*) AW(1)
      WRITE(*,*) 'Free energy for nucleation (J/mole)?'
      READ(*,*) AW(2)
      WRITE(*,*) 'Austenite strength at temperature (MPa)?'
      READ(*,*)  AW(3)
      CALL WEIGH(AMIN,AMAX,THETA1,THETA2,W1,W2)
      CALL  THICK(AW,W1,W2,THETA1,THETA2,AMIN,AMAX)
      STOP
      END


C********************************************************************
C To read the weights obtained from the neural network
C representation of bainite plate thickness
C H K D H Bhadeshia, University of Cambridge
C
C
       SUBROUTINE WEIGH(AMIN,AMAX,THETA1,THETA2,W1,W2)
       IMPLICIT REAL*8(A-H,K-Z), INTEGER(I,J)
       DOUBLE PRECISION W1(4,14), W2(4),THETA1(14),
     &  AMIN(15),AMAX(15)
C
C
           AVER=1.1
C
C
C Set number of inputs
           IN=3
C
C Set number of hidden units
           IHID=4
C
       OPEN(UNIT=2, FILE='WEIGHTS')
       DO 20 I=1,IN+1
         READ(2,*) AMIN(I),AMAX(I)
20     CONTINUE
C
          DO 2 I=1,IHID
            READ(2,*) THETA1(I)
C           WRITE(*,91) THETA1(I)
              DO 3 J=1,IN
                 READ(2,*) W1(I,J)
C                WRITE(*,91) W1(I,J)
3             CONTINUE
2         CONTINUE
C
          READ(2,*) THETA2
C         WRITE(*,91) THETA2
             DO 21 I=1,IHID
               READ(2,*) W2(I)
C              WRITE(*,91) W2(I)
21            CONTINUE
C

      RETURN
      END
C*************************************************************************
C**********************************************************************
C
C
      SUBROUTINE THICK(AW,W1,W2,THETA1,THETA2,AMIN,AMAX)
      IMPLICIT REAL*8(A-H,K-Z), INTEGER(I,J)
      DOUBLE PRECISION X(15), W1(4,14), W2(4),THETA1(14), H(4),
     &  AMIN(15),AMAX(15),AW(15)
C
C Remember to set matrix dimensions according to number of inputs
C   and number of hidden units. 
C

C
           AVER=1.3
C
C Set error at +- 20% for 95 % error limits
C  units are in microns
C
           ERROR=0.03d+00
C
C Set number of inputs
           IN=3
C
C Set number of hidden units
           IHID=4
C
C
C Read a set of unnormalized inputs
C
C
           X(1)=AW(1)
           X(2)=AW(2)
           X(3)=AW(3)

         CALL WRITO(X)
C
C


          DO 1 I=1,IN
            X(I) = ((X(I) -AMIN(I))/(AMAX(I)-AMIN(I)))-0.5D+00
C            WRITE(*,90)X(I)

1         CONTINUE

C
C IERROR = 0  implies that all inputs are within the range of the
C  training dataset for the neural network. Thus, the 95% confidence
C  error bars amount to about +- 10%
C
C IERROR = 1 implies that some inputs are out of the range of the
C  training dataset for the neural network. The 95% confidence
C  error bars MAY then be greater than +-10%. 
C
      DO 11 J=1,IN
         IF(X(J) .LT. -0.5D+00 .OR. X(J) .GT. 0.5D+00) THEN
           IERROR=1
           GOTO 12
         ELSE
           IERROR=0
         ENDIF
11    CONTINUE
12    CONTINUE
C
C Read all weights, and theta parameters
C
C
C
C Do calculation of the normalized output value for the given set of
C  normalized input
C
C First equation
C
       DO 6 I=1,IHID
          A=0.0D+00
C
               DO 5 J=1,IN
                 A=A+W1(I,J)*X(J)
5              CONTINUE
C
          H(I)=DTANH(A+THETA1(I))
6      CONTINUE
C
C Second equation
C
       A=0.0D+00
       DO 7 I=1,IHID
          A=A+W2(I)*H(I)
7      CONTINUE
C
       Y = A+THETA2
       Y=(Y+0.5)*(AMAX(IN+1)-AMIN(IN+1))+AMIN(IN+1)
C
C Prevent toughness from acquiring a negative value
C
       IF(Y .LT. 0.0D+00)Y=0.0D+00
C
C
       IF(IERROR .EQ. 0) THEN
           WRITE(*,91) Y, ERROR
       ELSE
           WRITE(*,92) Y, ERROR
       ENDIF
C
C
C

90          FORMAT(F10.2)
91          FORMAT(
     &10X,' Thickness /microns     ',F10.2/
     &10X,' Error     /microns     ',F10.2/)
92          FORMAT(
     &10X,' Thickness /microns     ',F10.2/
     &10X,' Error     /microns   > ',F10.2/)


C
C

       RETURN
       END
C******************************************************************
C Printing for the neural network representation of bainite plate thickness.
C S B Singh and H K D H Bhadeshia, University of Cambridge
C
      SUBROUTINE WRITO(A)
C
      DOUBLE PRECISION A(15)
      WRITE(*,4) A(1),A(2),A(3)
     
C
C View input variables

4      FORMAT(//////
     &15X,'             ** BAINITE SUBUNIT THICKNESS **           '//
     &15X,'                   University of Cambridge  '//
     &15X,'          S. B. Singh and H. K. D. H. Bhadeshia  '////
     &10X,' Temperature       ',F6.0,'  C '/
     &10X,' Gmax              ',F6.0,'  J/mole '/
     &10X,' Yield Strength    ',F6.0,'  MPa '/
     &//)
      RETURN
      END
C
C********************************************************************
